package cc.fascinated.bat.features.base.profile;

import cc.fascinated.bat.common.Profile;
import cc.fascinated.bat.features.Feature;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
public class FeatureProfile extends Profile {
    private static final FeatureState DEFAULT_STATE = FeatureState.ENABLED;

    /**
     * The feature states
     */
    private Map<String, FeatureState> featureStates;

    public FeatureProfile() {
        super("feature");
    }

    /**
     * Gets the feature states
     *
     * @return the feature states
     */
    public FeatureState getFeatureState(Feature feature) {
        if (this.featureStates == null) {
            this.featureStates = new HashMap<>();
        }
        if (feature == null) {
            return DEFAULT_STATE;
        }
        String featureName = feature.getName().toUpperCase();
        if (!this.featureStates.containsKey(featureName)) {
            this.featureStates.put(featureName, DEFAULT_STATE);
        }
        return this.featureStates.get(featureName);
    }

    /**
     * Enables the feature
     *
     * @param feature the feature to enable
     */
    public void enableFeature(Feature feature) {
        this.setFeatureState(feature, FeatureState.ENABLED);
    }

    /**
     * Disables the feature
     *
     * @param feature the feature to disable
     */
    public void disableFeature(Feature feature) {
        this.setFeatureState(feature, FeatureState.DISABLED);
    }

    /**
     * Sets the feature state
     *
     * @param feature the feature to set the state for
     * @param state   the state to set
     */
    public void setFeatureState(Feature feature, FeatureState state) {
        if (this.featureStates == null) {
            this.featureStates = new HashMap<>();
        }
        this.featureStates.put(feature.getName().toUpperCase(), state);
    }

    @Override
    public void reset() {
        this.featureStates = null;
    }

    public enum FeatureState {
        ENABLED,
        DISABLED
    }
}
