package cc.fascinated.bat.features.namehistory.profile.user;

import cc.fascinated.bat.common.Profile;
import cc.fascinated.bat.features.namehistory.NameHistoryFeature;
import cc.fascinated.bat.features.namehistory.TrackedName;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
public class NameHistoryProfile extends Profile {
    private List<TrackedName> nameHistory;

    public NameHistoryProfile() {
        super("name-history");
    }

    /**
     * Gets the name history of the user
     *
     * @return the name history of the user
     */
    public List<TrackedName> getNameHistory() {
        if (this.nameHistory == null) {
            this.nameHistory = new LinkedList<>();
        }
        return nameHistory;
    }

    /**
     * Gets the name history of the user sorted
     *
     * @return the name history of the user sorted
     */
    public List<TrackedName> getNameHistorySorted() {
        return getNameHistory().stream().sorted((o1, o2) -> o2.getChangedDate().compareTo(o1.getChangedDate())).toList();
    }

    /**
     * Adds a name to the name history
     *
     * @param name the name to add
     */
    public void addName(String name) {
        if (this.nameHistory == null) {
            this.nameHistory = new LinkedList<>();
        }
        this.nameHistory.add(new TrackedName(name, new Date()));
        cleanup();
    }

    /**
     * Cleans up the name history
     * <p>
     * This will remove any names that are not within
     * the size limit of the name history
     * </p>
     */
    private void cleanup() {
        if (this.nameHistory.size() > NameHistoryFeature.NAME_HISTORY_SIZE) {
            List<TrackedName> trackedNames = new ArrayList<>(this.nameHistory);
            trackedNames.sort((o1, o2) -> o2.getChangedDate().compareTo(o1.getChangedDate()));
            this.nameHistory = trackedNames.subList(0, NameHistoryFeature.NAME_HISTORY_SIZE);
        }
    }

    @Override
    public void reset() {
        this.nameHistory = null;
    }
}
